@@ -10,23 +10,21 @@ import sys
 
 from behave.contrib.scenario_autoretry import patch_scenario_with_autoretry
 from behave.log_capture import capture
+from behave.model import ScenarioOutline
 from behave.runner import Context, ModelRunner
 
 from behavex import conf_mgr
 from behavex.conf_mgr import get_env, get_param
 from behavex.global_vars import global_vars
 from behavex.outputs import report_json, report_xml
-from behavex.outputs.report_utils import create_log_path
-from behavex.utils import (
-    LOGGING_CFG,
-    create_custom_log_when_called,
-    get_autoretry_attempts,
-    get_logging_level,
-)
+from behavex.outputs.report_utils import create_log_path, strip_ansi_codes
+from behavex.utils import (LOGGING_CFG, create_custom_log_when_called,
+                           get_autoretry_attempts, get_logging_level,
+                           get_scenario_tags, get_scenarios_instances)
 
 Context.__getattribute__ = create_custom_log_when_called
-hooks_already_set = False
 
+hooks_already_set = False
 
 def extend_behave_hooks():
     """
@@ -35,33 +33,64 @@ def extend_behave_hooks():
     global hooks_already_set
     behave_run_hook = ModelRunner.run_hook
     behavex_env = sys.modules[__name__]
+    is_dry_run = get_param('dry_run')
 
     def run_hook(self, name, context, *args):
-        behave_run_hook(self, name, context, *args)
+
         if name == 'before_all':
             # noinspection PyUnresolvedReferences
+            if not is_dry_run:
+                behave_run_hook(self, name, context, *args)
             behavex_env.before_all(context)
-        if name == 'before_feature':
+        elif name == 'before_feature':
             # noinspection PyUnresolvedReferences
+            if not is_dry_run:
+                behave_run_hook(self, name, context, *args)
             behavex_env.before_feature(context, *args)
-        if name == 'before_scenario':
+        elif name == 'before_scenario':
             # noinspection PyUnresolvedReferences
+            if not is_dry_run:
+                behave_run_hook(self, name, context, *args)
             behavex_env.before_scenario(context, *args)
-        if name == 'before_step':
+        elif name == 'before_step':
             # noinspection PyUnresolvedReferences
+            if not is_dry_run:
+                behave_run_hook(self, name, context, *args)
             behavex_env.before_step(context, *args)
-        if name == 'after_step':
+        elif name == 'before_tag':
+            # noinspection PyUnresolvedReferences
+            if not is_dry_run:
+                behave_run_hook(self, name, context, *args)
+            behavex_env.before_tag(context, *args)
+        elif name == 'after_tag':
+            # noinspection PyUnresolvedReferences
+            if not is_dry_run:
+                behave_run_hook(self, name, context, *args)
+            behavex_env.after_tag(context, *args)
+        elif name == 'after_step':
             # noinspection PyUnresolvedReferences
             behavex_env.after_step(context, *args)
-        if name == 'after_scenario':
+            if not is_dry_run:
+                behave_run_hook(self, name, context, *args)
+        elif name == 'after_scenario':
             # noinspection PyUnresolvedReferences
+            if not is_dry_run:
+                behave_run_hook(self, name, context, *args)
             behavex_env.after_scenario(context, *args)
-        if name == 'after_feature':
+        elif name == 'after_feature':
             # noinspection PyUnresolvedReferences
             behavex_env.after_feature(context, *args)
-        if name == 'after_all':
+            if not is_dry_run:
+                behave_run_hook(self, name, context, *args)
+        elif name == 'after_all':
             # noinspection PyUnresolvedReferences
             behavex_env.after_all(context, *args)
+            if not is_dry_run:
+                behave_run_hook(self, name, context, *args)
+        else:
+            # noinspection PyUnresolvedReferences
+            if not is_dry_run:
+                behave_run_hook(self, name, context, *args)
 
     if not hooks_already_set:
         hooks_already_set = True
@@ -73,9 +102,9 @@ def before_all(context):
     try:
         # Initialyzing log handler
         context.bhx_log_handler = None
+        context.bhx_inside_scenario = False
         # Store framework settings to make them accessible from steps
         context.bhx_config_framework = conf_mgr.get_config()
-        context.bhx_inside_scenario = False
     except Exception as exception:
         _log_exception_and_continue('before_all (behavex)', exception)
 
@@ -84,13 +113,14 @@ def before_all(context):
 def before_feature(context, feature):
     try:
         context.bhx_execution_attempts = {}
-        for scenario in feature.scenarios:
-            scenario.tags += feature.tags
+        scenarios_instances = get_scenarios_instances(feature.scenarios)
+        for scenario in scenarios_instances:
+            scenario_tags = get_scenario_tags(scenario)
             if get_param('dry_run'):
-                if 'MANUAL' not in scenario.tags:
+                if 'MANUAL' not in scenario_tags:
                     scenario.tags.append(u'BHX_MANUAL_DRY_RUN')
                     scenario.tags.append(u'MANUAL')
-            configured_attempts = get_autoretry_attempts(scenario.tags)
+            configured_attempts = get_autoretry_attempts(scenario_tags)
             if configured_attempts > 0:
                 patch_scenario_with_autoretry(scenario, configured_attempts)
     except Exception as exception:
@@ -105,33 +135,42 @@ def before_scenario(context, scenario):
             context.bhx_execution_attempts[scenario.name] = 0
         execution_attempt = context.bhx_execution_attempts[scenario.name]
         retrying_execution = True if execution_attempt > 0 else False
-        concat_feature_and_scenario_name = "{}-{}".format(str(context.feature.name), str(scenario.name))
-        context.log_path = create_log_path(concat_feature_and_scenario_name, retrying_execution)
+        concat_feature_and_scenario_line = "{}-{}".format(str(context.feature.filename), str(scenario.line))
+        context.log_path = create_log_path(concat_feature_and_scenario_line, retrying_execution)
         context.bhx_log_handler = _add_log_handler(context.log_path)
         if retrying_execution:
-            logging.info('Retrying scenario...\n'.format())
+            logging.info('Retrying scenario execution...\n'.format())
             shutil.rmtree(context.evidence_path)
     except Exception as exception:
         _log_exception_and_continue('before_scenario (behavex)', exception)
 
 
 def before_step(context, step):
-    if context.bhx_inside_scenario:
-        logging.info('STEP: {}'.format(step.name))
+    pass
+
+
+def before_tag(context, tag):
+    pass
+
+
+def after_tag(context, tag):
+    pass
 
 
 def after_step(context, step):
-    if step.exception:
-        step.error_message = step.error_message
-        logging.exception(step.exception)
-    if context.bhx_inside_scenario:
-        logging.debug('Execution time %.2f sec\n', step.duration)
+    try:
+        if step.exception:
+            step.error_message = step.error_message
+            logging.error(step.exception)
+    except Exception as exception:
+        _log_exception_and_continue('after_step (behavex)', exception)
 
 
 @capture
 def after_scenario(context, scenario):
     try:
-        configured_attempts = get_autoretry_attempts(scenario.tags)
+        scenario_tags = get_scenario_tags(scenario)
+        configured_attempts = get_autoretry_attempts(scenario_tags)
         if scenario.status in ('failed', 'untested') and configured_attempts > 0:
             feature_name = scenario.feature.name
             if feature_name not in global_vars.retried_scenarios:
@@ -146,15 +185,19 @@ def after_scenario(context, scenario):
 
 # noinspection PyUnusedLocal
 def after_feature(context, feature):
-    if get_env('multiprocessing') and get_param('parallel_scheme') == 'scenario':
-        return
-    report_xml.export_feature_to_xml(feature)
+    try:
+        if get_env('multiprocessing') and get_param('parallel_scheme') == 'scenario':
+            return
+        report_xml.export_feature_to_xml(feature)
+    except Exception as exception:
+        _log_exception_and_continue('after_feature (behavex)', exception)
 
 
 def after_all(context):
     try:
         # noinspection PyProtectedMember
-        report_json.generate_execution_info(context, context._runner.features)
+        feature_list = report_json.generate_execution_info(context._runner.features)
+        report_json.generate_json_report(feature_list)
     except Exception as exception:
         _log_exception_and_continue('after_all (json_report)', exception)
 
@@ -167,6 +210,7 @@ def _add_log_handler(log_path):
     )
     log_level = get_logging_level()
     logging.getLogger().setLevel(log_level)
+    file_handler.addFilter(lambda record: setattr(record, 'msg', strip_ansi_codes(str(record.msg))) or True)
     file_handler.setFormatter(_get_log_formatter())
     logging.getLogger().addHandler(file_handler)
     return file_handler
@@ -199,5 +243,5 @@ def _log_exception_and_continue(module, exception):
     just to avoid the testing framework to stop executing
     the following tests"""
     error_message = "Unexpected error in '%s' function:" % module
-    logging.exception(error_message)
-    logging.exception(exception)
+    logging.error(error_message)
+    logging.error(exception)
