@@ -97,30 +97,6 @@ def get_summary_definition(steps):
     return result
 
 
-def count_by_status(values, func='getitem'):
-    if func == 'getitem':
-        get_item = getitem
-    elif func == 'getattr':
-        get_item = getattr
-    elif func == 'first_value':
-
-        def get_item(x, y):
-            return x
-
-    else:
-        get_item = func
-    skipped = len(
-        [
-            1
-            for value in values
-            if get_item(value, 'status') in ('skipped', 'undefined', 'untested')
-        ]
-    )
-    passed = len([1 for value in values if get_item(value, 'status') == 'passed'])
-    failed = len([1 for value in values if get_item(value, 'status') == 'failed'])
-    return passed, failed, skipped
-
-
 def calculate_status(list_status):
     set_status = set(list_status)
     if 'untested' in set_status:
@@ -171,31 +147,24 @@ def gather_steps(features):
 
 
 def gather_errors(scenario, retrieve_step_name=False):
+    error_msg = list(map(lambda line: strip_ansi_codes(line), scenario['error_msg']))
     if retrieve_step_name:
-        return scenario['error_msg'], scenario['error_lines'], scenario['error_step']
+        return error_msg, scenario['error_lines'], scenario['error_step']
     else:
-        return scenario['error_msg'], scenario['error_lines']
+        return error_msg, scenario['error_lines']
 
 
 def pretty_print_time(seconds_float, sec_decimals=1):
     seconds_float = round(seconds_float, sec_decimals)
-    minutes, seconds = divmod(seconds_float, 60)
-    hours, minutes = divmod(minutes, 60)
-
+    hours, remainder = divmod(seconds_float, 3600)
+    minutes, seconds = divmod(remainder, 60)
     seconds = (
         int(seconds) if float(seconds).is_integer() else round(seconds, sec_decimals)
     )
-
     def _pretty_format(cant, unit):
-        return '{}{}'.format(cant, unit) if cant > 0 else ''
-
-    time_string = '{} {} {}'.format(
-        _pretty_format(int(hours), 'h'),
-        _pretty_format(int(minutes), 'm'),
-        _pretty_format(seconds, 's'),
-    )
-    time_string = '0s' if time_string.strip() == '' else time_string
-    return time_string
+        return f'{cant}{unit}' if cant > 0 else ''
+    time_string = f'{_pretty_format(int(hours), "h")} {_pretty_format(int(minutes), "m")} {_pretty_format(seconds, "s")}'
+    return "0s" if time_string.strip() == '' else time_string
 
 
 def normalize_path(path):
@@ -221,7 +190,23 @@ def resolving_type(step, scenario, background=True):
             return step['step_type'].title()
 
 
-def try_operate_descriptor(dest_path, execution, return_value=False):
+def retry_file_operation(dest_path, execution, return_value=False):
+    """Executes a file operation with retry logic for handling file access conflicts.
+
+    This function attempts to execute a file operation (like copy, delete, etc.) and
+    provides an interactive retry mechanism if the files are locked by another process.
+
+    Args:
+        dest_path (str): The file system path where the operation will be performed
+        operation (callable): A function containing the file operation to execute
+        return_value (bool, optional): Whether to return the result of the operation. Defaults to False.
+
+    Returns:
+        Any: The result of the operation if return_value is True, otherwise None
+
+    Example:
+        >>> retry_file_operation('/path/to/dir', lambda: shutil.rmtree('/path/to/dir'))
+    """
     passed = False
     following = True
     retry_number = 1
@@ -281,7 +266,7 @@ def get_test_execution_tags():
         behave_tags = behave_tags_file.readline().strip()
         behave_tags_file.close()
         os.chmod(behave_tags_path, 511)  # nosec
-        set_env('behave_tags', behave_tags.replace('@MANUAL', 'False'))
+        set_env('behave_tags', behave_tags)
         return get_env('behave_tags')
     else:
         return get_env('behave_tags')
@@ -295,6 +280,7 @@ def match_for_execution(tags):
     if get_param('dry_run'):
         if 'BHX_MANUAL_DRY_RUN' in tags:
             tags.remove('BHX_MANUAL_DRY_RUN')
+        if 'MANUAL' in tags:
             tags.remove('MANUAL')
     # Set scenario tags in filter
     for tag in tags:
@@ -312,7 +298,7 @@ def match_for_execution(tags):
     for tag in tag_re.findall(tags_filter):
         if tag not in ('not', 'and', 'or', 'True', 'False'):
             tags_filter = tags_filter.replace(tag + ' ', 'False ')
-    return tags_filter == '' or eval(tags_filter)
+    return tags_filter == '' or eval(tags_filter)  # nosec
 
 
 def copy_bootstrap_html_generator(output):
@@ -320,8 +306,8 @@ def copy_bootstrap_html_generator(output):
     bootstrap_path = ['outputs', 'bootstrap']
     bootstrap_path = os.path.join(global_vars.execution_path, *bootstrap_path)
     if os.path.exists(dest_path):
-        try_operate_descriptor(dest_path, lambda: shutil.rmtree(dest_path))
-    try_operate_descriptor(
+        retry_file_operation(dest_path, lambda: shutil.rmtree(dest_path))
+    retry_file_operation(
         dest_path, lambda: shutil.copytree(bootstrap_path, dest_path)
     )
 
@@ -346,7 +332,7 @@ def get_save_function(path, content):
 
 
 def create_log_path(name, execution_retry=False):
-    name = normalize_filename(name)
+    name = get_string_hash(name)
     if sys.version_info < (3, 0):
         path = os.path.join(get_env('logs').decode('utf8'), str(name))
     else:
@@ -358,27 +344,30 @@ def create_log_path(name, execution_retry=False):
             return path
         path = initial_path + u'_' + str(scenario_outline_index)
         scenario_outline_index += 1
-    os.makedirs(path)
+    try:
+        os.makedirs(path)
+    except FileExistsError:
+        # path already exists
+        pass
     return path
 
 
-def get_error_message(message_error):
-    if not message_error:
+def get_error_message(error_message):
+    if not error_message:
         return u''
-    if isinstance(message_error, Exception):
-        if hasattr(message_error, 'message'):
+    if isinstance(error_message, Exception):
+        if hasattr(error_message, 'message'):
             # noinspection PyBroadException
             try:
-                message_error = traceback.format_tb(message_error.message)
+                error_message = traceback.format_tb(error_message.message)
             except BaseException:
-                message_error = repr(message_error.message)
-        if hasattr(message_error, 'exc_traceback'):
-            message_error = traceback.format_tb(message_error.exc_traceback)
-    elif not isinstance(message_error, str):
-        message_error = repr(message_error)
-    return u'\n'.join(
-        [16 * u' ' + line for line in text(message_error).split('\n')]
-    ).strip()
+                error_message = repr(error_message.message)
+        if hasattr(error_message, 'exc_traceback'):
+            error_message = traceback.format_tb(error_message.exc_traceback)
+    elif not isinstance(error_message, str):
+        error_message = repr(error_message)
+    formatted_error = u'\n'.join([16 * u' ' + line for line in text(error_message).split('\n')]).strip()
+    return formatted_error
 
 
 def text(value, encoding=None, errors=None):
@@ -409,6 +398,74 @@ def text(value, encoding=None, errors=None):
         return output_text
 
 
-def normalize_filename(input_name):
-    return str(hashlib.sha256(input_name.encode('UTF-8')).hexdigest())
+def get_string_hash(string_to_hash):
+    return str(hashlib.sha256(string_to_hash.encode('UTF-8')).hexdigest())
 
+
+def normalize_filename(input_name):
+    valid_name = None
+    try:
+        if sys.version_info[0] == 2:
+            if isinstance(input_name, str):
+                if isinstance(input_name, str):
+                    input_name = text(input_name)
+            else:
+                raise TypeError
+            if not input_name:
+                return u''
+            # xrange has been replaced for range
+            characters = ''.join(map(chr, range(255)))
+            reserved_characters = '<>:\'"/\\|?*'
+            for character in reserved_characters:
+                characters = characters.replace(character, '')
+            valid_name = ''.join(char for char in input_name if char in characters)
+            valid_name = unicodedata.normalize('NFKD', valid_name).encode(
+                'ascii', 'ignore'
+            )
+            valid_name = string.capwords(valid_name)
+            valid_name = valid_name.replace(' ', '_')
+            if isinstance(valid_name, str):
+                valid_name = text(valid_name)
+        if sys.version_info[0] == 3:
+            if isinstance(input_name, str):
+                pass
+            else:
+                raise TypeError
+            characters = ''.join(chr(i) for i in range(255))
+            reserved_characters = '<>:\'"/\\|?*'
+            for character in reserved_characters:
+                characters = characters.replace(character, '')
+            valid_name = ''.join(char for char in input_name if char in characters)
+            valid_name = unicodedata.normalize('NFKD', valid_name).encode(
+                'ascii', 'ignore'
+            )
+            if isinstance(valid_name, str):
+                valid_name = string.capwords(valid_name)
+            else:
+                valid_name = bytes.title(valid_name)
+            if isinstance(valid_name, str):
+                valid_name = valid_name.replace(' ', '_')
+            else:
+                valid_name = valid_name.replace(b' ', b'_')
+            if isinstance(valid_name, str):
+                valid_name = bytes(valid_name, encoding='UTF-8')
+        if isinstance(valid_name, bytes):
+            valid_name = valid_name.decode('utf8')
+        return valid_name
+    except TypeError:
+        err_msg = 'Input should be str or unicode, but received a: {}'.format(
+            type(input_name)
+        )
+        print(err_msg)
+        return
+
+
+def get_environment_details():
+    """ Get environment information from the ENVIRONMENT_DETAILS environment variable.
+    The environment information is a comma-separated string that is split into a list."""
+    environment_details_raw_data = os.getenv('ENVIRONMENT_DETAILS', None)
+    environment_details = environment_details_raw_data.split(',') if environment_details_raw_data else []
+    return environment_details
+
+def strip_ansi_codes(from_str: str):
+    return re.sub(r'\x1B\[[0-?9;]*[mGJK]', '', from_str)
