@@ -9,6 +9,8 @@ from __future__ import absolute_import, print_function
 
 import codecs
 import functools
+import glob
+import hashlib
 import json
 import logging
 import multiprocessing
@@ -16,6 +18,8 @@ import os
 import re
 import shutil
 import sys
+import time
+import uuid
 from functools import reduce
 from tempfile import gettempdir
 
@@ -28,13 +32,9 @@ from behavex.execution_singleton import ExecutionSingleton
 from behavex.global_vars import global_vars
 from behavex.outputs import report_html
 from behavex.outputs.output_strings import TEXTS
-from behavex.outputs.report_utils import (
-    get_save_function,
-    match_for_execution,
-    normalize_filename,
-    get_string_hash,
-    try_operate_descriptor,
-)
+from behavex.outputs.report_utils import (get_save_function, get_string_hash,
+                                          match_for_execution,
+                                          retry_file_operation)
 
 LOGGING_CFG = ConfigObj(os.path.join(global_vars.execution_path, 'conf_logging.cfg'))
 LOGGING_LEVELS = {
@@ -44,15 +44,33 @@ LOGGING_LEVELS = {
     'error': logging.ERROR,
     'critical': logging.CRITICAL,
 }
+ELAPSED_START_TIME = time.time()
 
 
-def append_results(codes, json_reports, tuple_values):
-    codes.append(tuple_values[0])
-    json_reports.append(tuple_values[1])
+def handle_execution_complete_callback(codes,
+                                       json_reports,
+                                       progress_bar_instance,
+                                       future):
+    tuple_values = None
+    try:
+        tuple_values = future.result()
+    except: # isort:skip
+        json_reports += []
+        codes.append(1)
+    if tuple_values:
+        execution_code, map_json = tuple_values
+        json_reports += [map_json]
+        codes.append(execution_code)
+    if progress_bar_instance:
+        progress_bar_instance.update()
 
 
-def create_partial_function_append(codes, json_reports):
-    return functools.partial(append_results, codes, json_reports)
+def create_execution_complete_callback_function(codes,
+                                                json_reports,
+                                                progress_bar_instance):
+    append_output = functools.partial(handle_execution_complete_callback,
+                                      codes, json_reports, progress_bar_instance)
+    return append_output
 
 
 def get_logging_level():
@@ -66,26 +84,25 @@ def get_logging_level():
 
 # noinspection PyDictCreation
 def join_feature_reports(json_reports):
-    scenario_lines = get_env('scenario_lines')
-    if len(json_reports) == 1:
-        merged_json = json_reports[0]
+    if type(json_reports) is list:
+        if len(json_reports) == 1:
+            merged_json = json_reports[0]
+        else:
+            merged_json = {}
+            merged_json['environment'] = join_list_dict(json_reports, 'environment')
+            merged_json['steps_definition'] = join_step_definitions(json_reports)
+            merged_json['features'] = sum((json_['features'] for json_ in json_reports), [])
     else:
-        merged_json = {}
-        merged_json['environment'] = join_list_dict(json_reports, 'environment')
-        merged_json['steps_definition'] = join_step_definitions(json_reports)
-        merged_json['features'] = sum((json_['features'] for json_ in json_reports), [])
-    if IncludeNameMatch().bool() or IncludePathsMatch().bool() or MatchInclude().bool():
+        merged_json = json_reports
+    if merged_json['features'] and (IncludeNameMatch().bool() or IncludePathsMatch().bool() or MatchInclude().bool()):
         delete = []
         for index, feature in enumerate(merged_json['features'][:]):
-            lines = scenario_lines.get(feature['filename'], {})
             scenarios = [
                 scenario
                 for scenario in feature['scenarios']
                 if IncludeNameMatch()(scenario['name'])
                 and MatchInclude()(feature['filename'])
-                and IncludePathsMatch()(
-                    scenario['filename'], lines.get(scenario['name'], -1)
-                )
+                and IncludePathsMatch()(scenario['filename'], scenario['line'])
             ]
             if not scenarios:
                 # create index list for delete after iterated the feature list.
@@ -161,38 +178,54 @@ def join_scenario_reports(json_reports):
 def explore_features(features_path, features_list=None):
     if features_list is None:
         features_list = []
-    for node in os.listdir(features_path):
-        if os.path.isdir(os.path.join(features_path, node)):
-            explore_features(os.path.join(features_path, node), features_list)
-        else:
-            if node.endswith('.feature'):
-                path_feature = os.path.abspath(os.path.join(features_path, node))
-                feature = should_be_run(path_feature)
-                if feature:
-                    features_list.append(feature)
+
+    # Normalize path separators
+    pure_feature_path, scenario_line = get_feature_and_scenario_line(features_path)
+    normalized_features_path = os.path.normpath(pure_feature_path)
+    if os.path.isfile(normalized_features_path):
+        if normalized_features_path.endswith('.feature'):
+            abs_feature_path = os.path.abspath(normalized_features_path)
+            feature = should_feature_be_run(abs_feature_path)
+            if feature:
+                if scenario_line:
+                    # iterate over scenarios and add the scenario that matches the scenario line
+                    for scenario in feature.scenarios:
+                        if scenario.line == int(scenario_line):
+                            features_list.append(scenario)
+                else:
+                    features_list.extend(feature.scenarios)
+    else:
+        try:
+            for node in os.listdir(normalized_features_path):
+                node_path = os.path.join(normalized_features_path, node)
+                if os.path.isdir(node_path):
+                    explore_features(node_path, features_list)
+                elif node.endswith('.feature'):
+                    abs_feature_path = os.path.abspath(node_path)
+                    feature = should_feature_be_run(abs_feature_path)
+                    if feature:
+                        features_list.extend(feature.scenarios)
+        except OSError as e:
+            print(f"Error accessing path {features_path}: {e}")
+            return features_list
+
     return features_list
 
 
-def should_be_run(path_feature):
+def should_feature_be_run(path_feature):
     feature = parse_file(path_feature)
     if not feature:
-        tags_list = []
+        return False
     else:
-        tags_list = [
-            scenario.tags
-            for scenario in feature.scenarios
-            if hasattr(feature, 'scenarios')
-        ]
-        tags_list.append(feature.tags)
+        tags_list = []
+        if hasattr(feature, 'scenarios'):
+            scenarios_instances = get_scenarios_instances(feature.scenarios)
+            for scenario in scenarios_instances:
+                scenario_tags = get_scenario_tags(scenario)
+                tags_list.append(scenario_tags)
     match_tag = any(match_for_execution(tags) for tags in tags_list)
-
     filename = feature.filename
-    if (
-        match_tag
-        and MatchInclude()(filename)
-        and match_any_paths(feature)
-        and match_any_name(feature)
-    ):
+    if match_tag and MatchInclude()(filename) and match_any_paths(feature) and match_any_name(feature):
         return feature
     else:
         return False
@@ -231,10 +264,10 @@ def copy_bootstrap_html_generator():
     bootstrap_path = ['outputs', 'bootstrap']
     bootstrap_path = os.path.join(global_vars.execution_path, *bootstrap_path)
     if os.path.exists(destination_path):
-        try_operate_descriptor(
+        retry_file_operation(
             destination_path, lambda: shutil.rmtree(destination_path)
         )
-    try_operate_descriptor(
+    retry_file_operation(
         destination_path, lambda: shutil.copytree(bootstrap_path, destination_path)
     )
 
@@ -246,18 +279,18 @@ def cleanup_folders():
     def execution():
         return shutil.rmtree(output_folder, ignore_errors=True)
 
-    try_operate_descriptor(output_folder, execution)
+    retry_file_operation(output_folder, execution)
     if not os.path.exists(output_folder):
-        try_operate_descriptor(output_folder, lambda: os.makedirs(output_folder))
+        retry_file_operation(output_folder, lambda: os.makedirs(output_folder))
     # temp folder
     temp_folder = get_env('temp')
 
     def execution():
         return shutil.rmtree(temp_folder, ignore_errors=True)
 
-    try_operate_descriptor(temp_folder, execution)
+    retry_file_operation(temp_folder, execution)
     if not os.path.exists(temp_folder):
-        try_operate_descriptor(temp_folder, lambda: os.makedirs(temp_folder))
+        retry_file_operation(temp_folder, lambda: os.makedirs(temp_folder))
 
     # behave folder
     behave_folder = os.path.join(get_env('OUTPUT'), 'behave')
@@ -265,9 +298,15 @@ def cleanup_folders():
     def execution():
         return shutil.rmtree(behave_folder, ignore_errors=True)
 
-    try_operate_descriptor(behave_folder, execution)
+    retry_file_operation(behave_folder, execution)
     if not os.path.exists(behave_folder):
-        try_operate_descriptor(behave_folder, lambda: os.makedirs(behave_folder))
+        retry_file_operation(behave_folder, lambda: os.makedirs(behave_folder))
+
+
+
+# Implemented for backward compatibility with other libraries that use this function
+def try_operate_descriptor(dest_path, execution, return_value=False):
+    return retry_file_operation(dest_path, execution, return_value)
 
 
 def set_env_variable(key, value):
@@ -279,21 +318,17 @@ def set_env_variable(key, value):
 def print_env_variables(keys):
     key_length = 20
     value_length = 60
-    print('|{}| {}|'.format(''.ljust(key_length, '-'), ''.ljust(value_length, '-')))
-    print(
-        '|{}| {}|'.format(
-            'ENV. VARIABLE'.ljust(key_length), 'VALUE'.ljust(value_length)
-        )
-    )
-    print('|{}| {}|'.format(''.ljust(key_length, '-'), ''.ljust(value_length, '-')))
+    separator = '|{}| {}|'.format(''.ljust(key_length, '-'), ''.ljust(value_length, '-'))
+    header = '|{}| {}|'.format('ENV. VARIABLE'.ljust(key_length), 'VALUE'.ljust(value_length))
+
+    print(separator)
+    print(header)
+    print(separator)
     for key in keys:
-        print(
-            '|{}| {}|'.format(
-                key.upper().ljust(key_length),
-                str(os.environ.get(key)).ljust(value_length),
-            )
-        )
-    print('|{}| {}|'.format(''.ljust(key_length, '-'), ''.ljust(value_length, '-')))
+        value = os.environ.get(key)
+        value = value if value else '----'
+        print('|{}| {}|'.format(key.upper().ljust(key_length), str(value).ljust(value_length)))
+    print(separator)
 
 
 def set_environ_config(args_parsed):
@@ -376,26 +411,13 @@ def len_scenarios(feature_file):
     data = codecs.open(feature_file, encoding='utf8').read()
     feature = parse_feature(data=data)
     amount_scenarios = 0
-    for scenario in feature.scenarios:
-        if match_for_execution(scenario.tags):
-            if isinstance(scenario, ScenarioOutline):
-                outline_instances = 1
-                for example in scenario.examples:
-                    rows = len(example.table.rows)
-                    if rows > outline_instances:
-                        outline_instances = rows
-                amount_scenarios += outline_instances
-            else:
-                amount_scenarios += 1
+    scenarios_instances = get_scenarios_instances(feature.scenarios)
+    for scenario in scenarios_instances:
+        if match_for_execution(get_scenario_tags(scenario)):
+            amount_scenarios += 1
     return amount_scenarios
 
 
-def check_environment_file():
-    path_environment = os.path.join(os.environ.get('FEATURES_PATH'), 'environment.py')
-    if not os.path.exists(path_environment):
-        raise Exception("environment.py module not found in 'features' folder")
-
-
 def set_behave_tags():
     behave_tags = os.path.join(get_env('OUTPUT'), 'behave', 'behave.tags')
     tags = []
@@ -420,11 +442,35 @@ def set_behave_tags():
     tags_line = ' '.join(tags)
     tags_line = tags_line.replace('~', 'not ')
     tags_line = tags_line.replace(',', ' or ')
-    try_operate_descriptor(
+    retry_file_operation(
         behave_tags, execution=get_save_function(behave_tags, tags_line)
     )
 
 
+def get_scenario_tags(scenario, include_outline_example_tags=True):
+    if type(scenario) is dict:
+        scenario_tags_set = set(scenario['tags'])
+    else:
+        scenario_tags = scenario.effective_tags
+        if include_outline_example_tags and isinstance(scenario, ScenarioOutline):
+            for example in scenario.examples:
+                scenario_tags.extend(example.tags)
+        scenario_tags_set = set(scenario_tags)
+    result = list(scenario_tags_set)
+    return result
+
+
+def get_scenarios_instances(scenarios):
+    scenarios_to_iterate = []
+    for scenario in scenarios:
+        if isinstance(scenario, ScenarioOutline):
+            for scenario_outline_instance in scenario.scenarios:
+                scenarios_to_iterate.append(scenario_outline_instance)
+        else:
+            scenarios_to_iterate.append(scenario)
+    return scenarios_to_iterate
+
+
 def set_system_paths():
     input_path = os.pathsep + get_env('temp')
     os.environ['PATH'] += input_path
@@ -440,7 +486,7 @@ def create_custom_log_when_called(self, key):
             if not hasattr(self, 'scenario'):
                 ex_msg = '"evidence_path" is only accessible in the context of a test scenario'
                 raise Exception(ex_msg)
-            self.log_path = get_string_hash("{}-{}".format(str(self.feature.name), str(self.scenario.name)))
+            self.log_path = get_string_hash("{}-{}".format(str(self.feature.filename), str(self.scenario.line)))
         evidence_path = os.path.join(self.log_path, 'evidence')
         self.evidence_path = evidence_path
         try:
@@ -487,33 +533,29 @@ class MatchInclude(metaclass=ExecutionSingleton):
 
 class IncludePathsMatch(metaclass=ExecutionSingleton):
     def __init__(self, paths=None):
-        if not paths:
-            paths = get_env('include_paths', get_param('include_paths'))
-        self.features_path = os.path.abspath(os.getenv('FEATURES_PATH'))
-        self.include_paths = [
-            os.path.abspath(path).replace(self.features_path, 'features')
-            for path in paths
-        ]
+        self.features_paths = os.getenv('FEATURES_PATH').split(",")
         self.features = [
-            path
-            for path in self.include_paths
-            if not os.path.isdir(path) and ':' not in path
+            os.path.abspath(path)
+            for path in self.features_paths
+            if os.path.isfile(path) and not has_scenario_line_number(path)
         ]
         self.scenarios = [
-            path
-            for path in self.include_paths
-            if not os.path.isdir(path) and ':' in path
+            "{}:{}".format(os.path.abspath(get_feature_and_scenario_line(path)[0]), get_feature_and_scenario_line(path)[1])
+            for path in self.features_paths
+            if not os.path.isdir(path) and has_scenario_line_number(path)
+        ]
+        self.folders = [
+            os.path.abspath(path)
+            for path in self.features_paths
+            if os.path.isdir(path) and not has_scenario_line_number(path)
         ]
-        self.folders = [path for path in paths if os.path.isdir(path)]
 
     def __call__(self, *args, **kwargs):
         return self.match(*args)
 
     def match(self, filename, scenario=None):
-        if not self.include_paths:
-            return True
+        filename = os.path.abspath(filename)
         match_scenario, match_feature = False, False
-        filename = os.path.abspath(filename).replace(self.features_path, 'features')
         if scenario:
             match_scenario = '{}:{}'.format(filename, scenario) in self.scenarios
         match_feature = filename in self.features
@@ -524,7 +566,8 @@ class IncludePathsMatch(metaclass=ExecutionSingleton):
         )
 
     def bool(self):
-        return self.include_paths and self.features and self.folders
+        return self.features and self.folders
+
 
 
 class IncludeNameMatch(metaclass=ExecutionSingleton):
@@ -552,3 +595,56 @@ def get_autoretry_attempts(tags):
             attempts_in_tag = result.group(2)
             attempts = int(attempts_in_tag) if attempts_in_tag else 2
     return attempts
+
+def generate_hash(word):
+    # Use SHA-256 for better distribution
+    sha256 = hashlib.sha256(word.encode('utf-8')).digest()
+    # Take the first 6 bytes (48 bits) of the hash
+    truncated_hash = sha256[:6]
+    # Convert to an integer
+    hash_int = int.from_bytes(truncated_hash, byteorder='big')
+    # Ensure the result fits in 48 bits (optional, for consistency)
+    return hash_int & 0xFFFFFFFFFFFF
+
+def generate_uuid():
+    return uuid.uuid4().hex
+
+def expand_paths(paths):
+    """Expand glob patterns in paths and verify they exist.
+
+    Args:
+        paths (list): List of paths that may contain glob patterns
+
+    Returns:
+        list: List of expanded and verified paths
+    """
+    expanded = []
+    for path in paths:
+        # Handle glob patterns
+        if any(char in path for char in ['*', '?', '[']):
+            globbed = glob.glob(path)
+            if not globbed:
+                print('\nNo files found matching pattern: {}'.format(path))
+                exit()
+            expanded.extend(globbed)
+        else:
+            pure_feature_path, scenario_line = get_feature_and_scenario_line(path)
+            normalized_features_path = os.path.normpath(pure_feature_path)
+            if not os.path.exists(pure_feature_path):
+                print('\nSpecified path was not found: {}'.format(pure_feature_path))
+                exit()
+            expanded.append(path)
+    return expanded
+
+
+def has_scenario_line_number(path):
+    # Split path by colon and check if the last part is a number
+    parts = path.split(':')
+    return len(parts) > 1 and parts[-1].isdigit()
+
+def get_feature_and_scenario_line(path):
+    if has_scenario_line_number(path):
+        parts = path.split(':')
+        return [':'.join(parts[:-1]), parts[-1]]
+    else:
+        return [path, None]
